#pragma once

// This file is not used by the runtime, instead, a version is generated during
// compilation with only the version for the current build. This is kept in its
// own file so that not all files have to be recompiled for each new release.
//
// This file is only used by static analyzers and IDEs.

#include "esphome/core/macros.h"

#define ESPHOME_VERSION "dev"
#define ESPHOME_VERSION_CODE VERSION_CODE(2099, 12, 0)
